/*
 * Copyright (C) 2013-2014 Pier Luigi Fiorini
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLASMA_SHELL_CLIENT_PROTOCOL_H
#define PLASMA_SHELL_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct org_kde_plasma_shell;
struct org_kde_plasma_surface;
struct wl_output;
struct wl_surface;

extern const struct wl_interface org_kde_plasma_shell_interface;
extern const struct wl_interface org_kde_plasma_surface_interface;

#define ORG_KDE_PLASMA_SHELL_GET_SURFACE	0

#define ORG_KDE_PLASMA_SHELL_GET_SURFACE_SINCE_VERSION	1

static inline void
org_kde_plasma_shell_set_user_data(struct org_kde_plasma_shell *org_kde_plasma_shell, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_plasma_shell, user_data);
}

static inline void *
org_kde_plasma_shell_get_user_data(struct org_kde_plasma_shell *org_kde_plasma_shell)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_plasma_shell);
}

static inline uint32_t
org_kde_plasma_shell_get_version(struct org_kde_plasma_shell *org_kde_plasma_shell)
{
	return wl_proxy_get_version((struct wl_proxy *) org_kde_plasma_shell);
}

static inline void
org_kde_plasma_shell_destroy(struct org_kde_plasma_shell *org_kde_plasma_shell)
{
	wl_proxy_destroy((struct wl_proxy *) org_kde_plasma_shell);
}

static inline struct org_kde_plasma_surface *
org_kde_plasma_shell_get_surface(struct org_kde_plasma_shell *org_kde_plasma_shell, struct wl_surface *surface)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) org_kde_plasma_shell,
			 ORG_KDE_PLASMA_SHELL_GET_SURFACE, &org_kde_plasma_surface_interface, NULL, surface);

	return (struct org_kde_plasma_surface *) id;
}

#ifndef ORG_KDE_PLASMA_SURFACE_ROLE_ENUM
#define ORG_KDE_PLASMA_SURFACE_ROLE_ENUM
enum org_kde_plasma_surface_role {
	ORG_KDE_PLASMA_SURFACE_ROLE_NORMAL = 0,
	ORG_KDE_PLASMA_SURFACE_ROLE_DESKTOP = 1,
	ORG_KDE_PLASMA_SURFACE_ROLE_PANEL = 2,
	ORG_KDE_PLASMA_SURFACE_ROLE_ONSCREENDISPLAY = 3,
	ORG_KDE_PLASMA_SURFACE_ROLE_NOTIFICATION = 4,
	ORG_KDE_PLASMA_SURFACE_ROLE_TOOLTIP = 5,
};
#endif /* ORG_KDE_PLASMA_SURFACE_ROLE_ENUM */

#ifndef ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_ENUM
#define ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_ENUM
/**
 * org_kde_plasma_surface_panel_behavior - Behavior for panel surface
 * @ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_ALWAYS_VISIBLE: normal panel
 *	visibility
 * @ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_AUTO_HIDE: hide automatically
 * @ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_WINDOWS_CAN_COVER: windows can
 *	cover
 * @ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_WINDOWS_GO_BELOW: windows go
 *	below
 *
 * 
 */
enum org_kde_plasma_surface_panel_behavior {
	ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_ALWAYS_VISIBLE = 1,
	ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_AUTO_HIDE = 2,
	ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_WINDOWS_CAN_COVER = 3,
	ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_WINDOWS_GO_BELOW = 4,
};
#endif /* ORG_KDE_PLASMA_SURFACE_PANEL_BEHAVIOR_ENUM */

#ifndef ORG_KDE_PLASMA_SURFACE_ERROR_ENUM
#define ORG_KDE_PLASMA_SURFACE_ERROR_ENUM
enum org_kde_plasma_surface_error {
	ORG_KDE_PLASMA_SURFACE_ERROR_PANEL_NOT_AUTO_HIDE = 0,
};
#endif /* ORG_KDE_PLASMA_SURFACE_ERROR_ENUM */

/**
 * org_kde_plasma_surface - metadata interface
 * @auto_hidden_panel_hidden: Auto-hiding panel is hidden
 * @auto_hidden_panel_shown: Auto-hiding panel is shown
 *
 * An interface that may be implemented by a wl_surface, for
 * implementations that provide the shell user interface.
 *
 * It provides requests to set surface roles, assign an output or set the
 * position in output coordinates.
 *
 * On the server side the object is automatically destroyed when the
 * related wl_surface is destroyed. On client side,
 * org_kde_plasma_surface.destroy() must be called before destroying the
 * wl_surface object.
 */
struct org_kde_plasma_surface_listener {
	/**
	 * auto_hidden_panel_hidden - Auto-hiding panel is hidden
	 *
	 * An auto-hiding panel got hidden by the compositor.
	 * @since: 4
	 */
	void (*auto_hidden_panel_hidden)(void *data,
					 struct org_kde_plasma_surface *org_kde_plasma_surface);
	/**
	 * auto_hidden_panel_shown - Auto-hiding panel is shown
	 *
	 * An auto-hiding panel got shown by the compositor.
	 * @since: 4
	 */
	void (*auto_hidden_panel_shown)(void *data,
					struct org_kde_plasma_surface *org_kde_plasma_surface);
};

static inline int
org_kde_plasma_surface_add_listener(struct org_kde_plasma_surface *org_kde_plasma_surface,
				    const struct org_kde_plasma_surface_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) org_kde_plasma_surface,
				     (void (**)(void)) listener, data);
}

#define ORG_KDE_PLASMA_SURFACE_DESTROY	0
#define ORG_KDE_PLASMA_SURFACE_SET_OUTPUT	1
#define ORG_KDE_PLASMA_SURFACE_SET_POSITION	2
#define ORG_KDE_PLASMA_SURFACE_SET_ROLE	3
#define ORG_KDE_PLASMA_SURFACE_SET_PANEL_BEHAVIOR	4
#define ORG_KDE_PLASMA_SURFACE_SET_SKIP_TASKBAR	5
#define ORG_KDE_PLASMA_SURFACE_PANEL_AUTO_HIDE_HIDE	6
#define ORG_KDE_PLASMA_SURFACE_PANEL_AUTO_HIDE_SHOW	7
#define ORG_KDE_PLASMA_SURFACE_SET_PANEL_TAKES_FOCUS	8

#define ORG_KDE_PLASMA_SURFACE_DESTROY_SINCE_VERSION	1
#define ORG_KDE_PLASMA_SURFACE_SET_OUTPUT_SINCE_VERSION	1
#define ORG_KDE_PLASMA_SURFACE_SET_POSITION_SINCE_VERSION	1
#define ORG_KDE_PLASMA_SURFACE_SET_ROLE_SINCE_VERSION	1
#define ORG_KDE_PLASMA_SURFACE_SET_PANEL_BEHAVIOR_SINCE_VERSION	1
#define ORG_KDE_PLASMA_SURFACE_SET_SKIP_TASKBAR_SINCE_VERSION	2
#define ORG_KDE_PLASMA_SURFACE_PANEL_AUTO_HIDE_HIDE_SINCE_VERSION	4
#define ORG_KDE_PLASMA_SURFACE_PANEL_AUTO_HIDE_SHOW_SINCE_VERSION	4
#define ORG_KDE_PLASMA_SURFACE_SET_PANEL_TAKES_FOCUS_SINCE_VERSION	4

static inline void
org_kde_plasma_surface_set_user_data(struct org_kde_plasma_surface *org_kde_plasma_surface, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) org_kde_plasma_surface, user_data);
}

static inline void *
org_kde_plasma_surface_get_user_data(struct org_kde_plasma_surface *org_kde_plasma_surface)
{
	return wl_proxy_get_user_data((struct wl_proxy *) org_kde_plasma_surface);
}

static inline uint32_t
org_kde_plasma_surface_get_version(struct org_kde_plasma_surface *org_kde_plasma_surface)
{
	return wl_proxy_get_version((struct wl_proxy *) org_kde_plasma_surface);
}

static inline void
org_kde_plasma_surface_destroy(struct org_kde_plasma_surface *org_kde_plasma_surface)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_DESTROY);

	wl_proxy_destroy((struct wl_proxy *) org_kde_plasma_surface);
}

static inline void
org_kde_plasma_surface_set_output(struct org_kde_plasma_surface *org_kde_plasma_surface, struct wl_output *output)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_SET_OUTPUT, output);
}

static inline void
org_kde_plasma_surface_set_position(struct org_kde_plasma_surface *org_kde_plasma_surface, int32_t x, int32_t y)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_SET_POSITION, x, y);
}

static inline void
org_kde_plasma_surface_set_role(struct org_kde_plasma_surface *org_kde_plasma_surface, uint32_t role)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_SET_ROLE, role);
}

static inline void
org_kde_plasma_surface_set_panel_behavior(struct org_kde_plasma_surface *org_kde_plasma_surface, uint32_t flag)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_SET_PANEL_BEHAVIOR, flag);
}

static inline void
org_kde_plasma_surface_set_skip_taskbar(struct org_kde_plasma_surface *org_kde_plasma_surface, uint32_t skip)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_SET_SKIP_TASKBAR, skip);
}

static inline void
org_kde_plasma_surface_panel_auto_hide_hide(struct org_kde_plasma_surface *org_kde_plasma_surface)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_PANEL_AUTO_HIDE_HIDE);
}

static inline void
org_kde_plasma_surface_panel_auto_hide_show(struct org_kde_plasma_surface *org_kde_plasma_surface)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_PANEL_AUTO_HIDE_SHOW);
}

static inline void
org_kde_plasma_surface_set_panel_takes_focus(struct org_kde_plasma_surface *org_kde_plasma_surface, uint32_t takes_focus)
{
	wl_proxy_marshal((struct wl_proxy *) org_kde_plasma_surface,
			 ORG_KDE_PLASMA_SURFACE_SET_PANEL_TAKES_FOCUS, takes_focus);
}

#ifdef  __cplusplus
}
#endif

#endif
