/*
 * Copyright (C) 2015 Martin Gräßlin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SERVER_DECORATION_SERVER_PROTOCOL_H
#define SERVER_DECORATION_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct org_kde_kwin_server_decoration;
struct org_kde_kwin_server_decoration_manager;
struct wl_surface;

extern const struct wl_interface org_kde_kwin_server_decoration_manager_interface;
extern const struct wl_interface org_kde_kwin_server_decoration_interface;

#ifndef ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_ENUM
#define ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_ENUM
/**
 * org_kde_kwin_server_decoration_manager_mode - Possible values to use
 *	in request_mode and the event mode.
 * @ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_NONE: Undecorated: The
 *	surface is not decorated at all, neither server nor client-side. An
 *	example is a popup surface which should not be decorated.
 * @ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_CLIENT: Client-side
 *	decoration: The decoration is part of the surface and the client.
 * @ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_SERVER: Server-side
 *	decoration: The server embeds the surface into a decoration frame.
 *
 * 
 */
enum org_kde_kwin_server_decoration_manager_mode {
	ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_NONE = 0,
	ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_CLIENT = 1,
	ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_SERVER = 2,
};
#endif /* ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_MODE_ENUM */

/**
 * org_kde_kwin_server_decoration_manager - Server side window decoration
 *	manager
 * @create: Create a server-side decoration object for a given surface
 *
 * This interface allows to coordinate whether the server should create a
 * server-side window decoration around a wl_surface representing a shell
 * surface (wl_shell_surface or similar). By announcing support for this
 * interface the server indicates that it supports server side decorations.
 */
struct org_kde_kwin_server_decoration_manager_interface {
	/**
	 * create - Create a server-side decoration object for a given
	 *	surface
	 * @id: (none)
	 * @surface: (none)
	 *
	 * When a client creates a server-side decoration object it
	 * indicates that it supports the protocol. The client is supposed
	 * to tell the server whether it wants server-side decorations or
	 * will provide client-side decorations.
	 *
	 * If the client does not create a server-side decoration object
	 * for a surface the server interprets this as lack of support for
	 * this protocol and considers it as client-side decorated.
	 * Nevertheless a client-side decorated surface should use this
	 * protocol to indicate to the server that it does not want a
	 * server-side deco.
	 */
	void (*create)(struct wl_client *client,
		       struct wl_resource *resource,
		       uint32_t id,
		       struct wl_resource *surface);
};

#define ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_DEFAULT_MODE	0

#define ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_DEFAULT_MODE_SINCE_VERSION	1

static inline void
org_kde_kwin_server_decoration_manager_send_default_mode(struct wl_resource *resource_, uint32_t mode)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_SERVER_DECORATION_MANAGER_DEFAULT_MODE, mode);
}

#ifndef ORG_KDE_KWIN_SERVER_DECORATION_MODE_ENUM
#define ORG_KDE_KWIN_SERVER_DECORATION_MODE_ENUM
/**
 * org_kde_kwin_server_decoration_mode - Possible values to use in
 *	request_mode and the event mode.
 * @ORG_KDE_KWIN_SERVER_DECORATION_MODE_NONE: Undecorated: The surface is
 *	not decorated at all, neither server nor client-side. An example is a
 *	popup surface which should not be decorated.
 * @ORG_KDE_KWIN_SERVER_DECORATION_MODE_CLIENT: Client-side decoration:
 *	The decoration is part of the surface and the client.
 * @ORG_KDE_KWIN_SERVER_DECORATION_MODE_SERVER: Server-side decoration:
 *	The server embeds the surface into a decoration frame.
 *
 * 
 */
enum org_kde_kwin_server_decoration_mode {
	ORG_KDE_KWIN_SERVER_DECORATION_MODE_NONE = 0,
	ORG_KDE_KWIN_SERVER_DECORATION_MODE_CLIENT = 1,
	ORG_KDE_KWIN_SERVER_DECORATION_MODE_SERVER = 2,
};
#endif /* ORG_KDE_KWIN_SERVER_DECORATION_MODE_ENUM */

struct org_kde_kwin_server_decoration_interface {
	/**
	 * release - release the server decoration object
	 *
	 * 
	 */
	void (*release)(struct wl_client *client,
			struct wl_resource *resource);
	/**
	 * request_mode - The decoration mode the surface wants to use.
	 * @mode: The mode this surface wants to use.
	 *
	 * 
	 */
	void (*request_mode)(struct wl_client *client,
			     struct wl_resource *resource,
			     uint32_t mode);
};

#define ORG_KDE_KWIN_SERVER_DECORATION_MODE	0

#define ORG_KDE_KWIN_SERVER_DECORATION_MODE_SINCE_VERSION	1

static inline void
org_kde_kwin_server_decoration_send_mode(struct wl_resource *resource_, uint32_t mode)
{
	wl_resource_post_event(resource_, ORG_KDE_KWIN_SERVER_DECORATION_MODE, mode);
}

#ifdef  __cplusplus
}
#endif

#endif
